// Example for Controlling Freshwater EC with Open Source Bio  - if controlling saltware increase the control loop amounts to the recommended levels. 
// A Switching Relay is connected to a distilled water pump, the relay is connected to pin 12 and Ground
// A Switching Relay is connected to an salty water pump, the relay is connected to pin 11 and Ground
// If you are only controlling with one pump the loop will still work, just hook up the correct pin

// The commands below are necessary for the OSB library to work
#include <Wire.h>
#include <Boekel_OpenSourceBio.h>
using namespace Boekel;
OpenSourceBio osb; // create osb object.


const int distilled = 12; 
const int salty = 11;

//The setup() function will only run once. 
void setup()
{
  delay(1500);  // The main board takes a second to warm up.  This ensures it is ready to communicate
  osb.clearScreen(OpenSourceBio::COLOR_BLACK); // clears user programmable screen 
  pinMode(distilled, OUTPUT); //Setting pin 12 "distilled" as an output 
  pinMode(salty, OUTPUT); //Setting pin 11 "salty" as an output 
}


//The loop() function will run for ever
void loop()
{

  osb.clearScreen(OpenSourceBio::COLOR_BLACK); // clears user programmable screen
  
  //Creating a varaiable: we will store the "reading" as a double varaible
  double reading;
  
  //The OSB library command below updates the readings from the Transmitter
  osb.updateReadings(); 
  
  //We are setting the reading varaible equal to EC value with the osb.getEC(); command in the OSB library  
  reading = osb.getEC();;  
   
  //Adds Salty water if the EC is less than or equal to 1000  (change to 45000 for saltwater)
   if (reading <= 1000)  
           {
            // Displays text on screen(x,y, text color, background color)
            osb.displayText(100, 100, OpenSourceBio::COLOR_BLUE, OpenSourceBio::COLOR_BLACK,"Adding Saltwater");
            
            digitalWrite(salty, HIGH); // Turns salt pump relay on 
            delay(5000); //Keep pump on for 5 seconds
            digitalWrite(salty, LOW); //Turn pump off
            osb.displayText(10, 10, OpenSourceBio::COLOR_BLUE, OpenSourceBio::COLOR_BLACK,"Pausing for 5 seconds");
            delay(5000); //Pauses the pump for 5 seconds to allow for mixing     
            
            }
  else if (reading >= 2000) // Adds distilled water if the EC is greater than or equal to 2000  (change to 55000 for saltwater)
           {
             
            // Displays text on screen(x,y, text color, background color)
            osb.displayText(100, 100, OpenSourceBio::COLOR_RED, OpenSourceBio::COLOR_BLACK,"Adding Distilled Water");
            
            digitalWrite(distilled, HIGH); // Turns distilled pump relay on 
            delay(5000); //Keep pump on for 5 seconds
            digitalWrite(distilled, LOW); //Turn pump off
            osb.displayText(10, 10, OpenSourceBio::COLOR_RED, OpenSourceBio::COLOR_BLACK,"Pausing for 5 seconds");
            delay(5000); //Pauses the pump for ten seconds to allow for mixing  
           } 
  else
  
          {
            osb.displayText(60, 100,OpenSourceBio::COLOR_GREEN, OpenSourceBio::COLOR_BLACK,"EC is within range");
            delay(2000); //Delays for 2 seconds
            
          }
  
}
